@@ -9,12 +9,12 @@
 
 //! Arbitrary implementations for std::borrow.
 
+use crate::std_facade::fmt;
+use crate::std_facade::{Cow, ToOwned};
 use core::borrow::Borrow;
-use std_facade::{ToOwned, Cow};
-use std_facade::fmt;
 
-use strategy::statics::static_map;
-use arbitrary::{any_with, SMapped, Arbitrary};
+use crate::arbitrary::{any_with, Arbitrary, SMapped};
+use crate::strategy::statics::static_map;
 
 arbitrary!(
     [A: Arbitrary + Borrow<B>, B: ToOwned<Owned = A> + fmt::Debug]
