@@ -1,5 +1,5 @@
 //-
-// Copyright 2017, 2018 The proptest developers
+// Copyright 2017, 2018, 2019 The proptest developers
 //
 // Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
 // http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
@@ -9,19 +9,20 @@
 
 use core::any::Any;
 use core::fmt::Debug;
-use core::num::ParseIntError;
 use std::borrow::{Cow, ToOwned};
 use std::boxed::Box;
 use std::env;
 use std::fs;
 use std::io::{self, BufRead, Write};
 use std::path::{Path, PathBuf};
+use std::string::{String, ToString};
 use std::sync::RwLock;
 use std::vec::Vec;
-use std::string::{String, ToString};
 
-use test_runner::{Seed, failure_persistence::FailurePersistence};
 use self::FileFailurePersistence::*;
+use crate::test_runner::failure_persistence::{
+    FailurePersistence, PersistedSeed,
+};
 
 /// Describes how failing test cases are persisted.
 ///
@@ -81,25 +82,31 @@ impl Default for FileFailurePersistence {
 }
 
 impl FailurePersistence for FileFailurePersistence {
-    fn load_persisted_failures(&self, source_file: Option<&'static str>)
-                               -> Vec<Seed> {
+    fn load_persisted_failures2(
+        &self,
+        source_file: Option<&'static str>,
+    ) -> Vec<PersistedSeed> {
         let p = self.resolve(
-            source_file.and_then(|s| absolutize_source_file(Path::new(s)))
+            source_file
+                .and_then(|s| absolutize_source_file(Path::new(s)))
                 .as_ref()
-                .map(|cow| &**cow));
+                .map(|cow| &**cow),
+        );
 
         let path: Option<&PathBuf> = p.as_ref();
-        let result: io::Result<Vec<Seed>> = path.map_or_else(
+        let result: io::Result<Vec<PersistedSeed>> = path.map_or_else(
             || Ok(vec![]),
             |path| {
                 // .ok() instead of .unwrap() so we don't propagate panics here
                 let _lock = PERSISTENCE_LOCK.read().ok();
                 io::BufReader::new(fs::File::open(path)?)
-                    .lines().enumerate()
+                    .lines()
+                    .enumerate()
                     .filter_map(|(lineno, line)| match line {
                         Err(err) => Some(Err(err)),
-                        Ok(line) => parse_seed_line(line, path, lineno).map(Ok)
-                    }).collect()
+                        Ok(line) => parse_seed_line(line, path, lineno).map(Ok),
+                    })
+                    .collect()
             },
         );
 
@@ -117,10 +124,10 @@ impl FailurePersistence for FileFailurePersistence {
         })
     }
 
-    fn save_persisted_failure(
+    fn save_persisted_failure2(
         &mut self,
         source_file: Option<&'static str>,
-        seed: Seed,
+        seed: PersistedSeed,
         shrunken_value: &dyn Debug,
     ) {
         let path = self.resolve(source_file.map(Path::new));
@@ -135,11 +142,15 @@ impl FailurePersistence for FileFailurePersistence {
                     .expect("proptest: couldn't write header.");
             }
 
-            write_seed_line(&mut to_write, seed, shrunken_value)
+            write_seed_line(&mut to_write, &seed, shrunken_value)
                 .expect("proptest: couldn't write seed line.");
 
             if let Err(e) = write_seed_data_to_file(&path, &to_write) {
-                eprintln!("proptest: failed to append to {}: {}", path.display(), e);
+                eprintln!(
+                    "proptest: failed to append to {}: {}",
+                    path.display(),
+                    e
+                );
             } else if is_new {
                 eprintln!(
                     "proptest: Saving this and future failures in {}\n\
@@ -148,7 +159,7 @@ impl FailurePersistence for FileFailurePersistence {
                      {}",
                     path.display(),
                     if is_new { " (You may need to create it.)" } else { "" },
-                    format_basic_seed_line(seed));
+                    seed);
             }
         }
     }
@@ -158,10 +169,15 @@ impl FailurePersistence for FileFailurePersistence {
     }
 
     fn eq(&self, other: &dyn FailurePersistence) -> bool {
-        other.as_any().downcast_ref::<Self>().map_or(false, |x| x == self)
+        other
+            .as_any()
+            .downcast_ref::<Self>()
+            .map_or(false, |x| x == self)
     }
 
-    fn as_any(&self) -> &dyn Any { self }
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
 }
 
 /// Ensure that the source file to use for resolving the location of the persisted
@@ -184,7 +200,7 @@ fn absolutize_source_file<'a>(source: &'a Path) -> Option<Cow<'a, Path>> {
 }
 
 fn absolutize_source_file_with_cwd<'a>(
-    getcwd: impl FnOnce () -> io::Result<PathBuf>,
+    getcwd: impl FnOnce() -> io::Result<PathBuf>,
     source: &'a Path,
 ) -> Option<Cow<'a, Path>> {
     if source.is_absolute() {
@@ -234,82 +250,42 @@ fn absolutize_source_file_with_cwd<'a>(
     }
 }
 
-fn parse_seed_line(mut line: String, path: &Path, lineno: usize)
-                   -> Option<Seed> {
+fn parse_seed_line(
+    mut line: String,
+    path: &Path,
+    lineno: usize,
+) -> Option<PersistedSeed> {
     // Remove anything after and including '#':
     if let Some(comment_start) = line.find('#') {
         line.truncate(comment_start);
     }
 
     if line.len() > 0 {
-        // Split by whitespace and ignore empty lines:
-        let parts = line.trim().split(char::is_whitespace).collect::<Vec<_>>();
-        let len = parts.len();
-        // "xs" stands for "XorShift".
-        if parts[0] == "xs" && len == 5 {
-            // Parse using the chosen one:
-            if let Ok(seed) = parse_seed_old(&parts[1..]) {
-                return Some(seed);
-            } else {
-                eprintln!("proptest: {}:{}: unparsable line, ignoring",
-                            path.display(), lineno + 1);
-            }
-        } else {
-            eprintln!("proptest: {}:{}: unknown case type `{}` \
-                    (corrupt file or newer proptest version?)",
-                    &path.display(), lineno + 1, parts[0]);
+        let ret = line.parse::<PersistedSeed>().ok();
+        if !ret.is_some() {
+            eprintln!(
+                "proptest: {}:{}: unparsable line, ignoring",
+                path.display(),
+                lineno + 1
+            );
         }
+        return ret;
     }
 
     None
 }
 
-fn parse_seed_old(parts: &[&str]) -> Result<Seed, ParseIntError> {
-    let mut ret = [0u32; 4];
-    for (src, dst) in parts.iter().zip(ret.iter_mut()) {
-        *dst = src.parse()?;
-    }
-
-    Ok(convert_to_new_format(ret))
-}
-
-fn convert_to_new_format(old_format: [u32; 4]) -> Seed {
-    use byteorder::{ByteOrder, LittleEndian};
-    let mut new_format = [0; 16];
-    // rand uses little endian for this conversion on all platforms
-    LittleEndian::write_u32_into(&old_format[..], &mut new_format);
-    new_format
-}
-
-fn convert_from_new_format(new_format: Seed) -> [u32; 4] {
-    use byteorder::{ByteOrder, LittleEndian};
-    let mut old_format = [0; 4];
-    LittleEndian::read_u32_into(&new_format[..], &mut old_format);
-    old_format
-}
-
-fn format_basic_seed_line(seed: Seed) -> String {
-    // Write line start:
-    let mut buf = "xs ".to_owned();
-
-    // Write out each part of seed:
-    for &s in &convert_from_new_format(seed) {
-        buf.push_str(&s.to_string());
-        buf.push(' ');
-    }
-
-    buf
-}
-
-fn write_seed_line(buf: &mut Vec<u8>, seed: Seed, shrunken_value: &dyn Debug)
-    -> io::Result<()>
-{
+fn write_seed_line(
+    buf: &mut Vec<u8>,
+    seed: &PersistedSeed,
+    shrunken_value: &dyn Debug,
+) -> io::Result<()> {
     // Write the seed itself
-    write!(buf, "{}", format_basic_seed_line(seed));
+    write!(buf, "{}", seed.to_string())?;
 
     // Write out comment:
     let debug_start = buf.len();
-    write!(buf, "# shrinks to {:?}", shrunken_value)?;
+    write!(buf, " # shrinks to {:?}", shrunken_value)?;
 
     // Ensure there are no newlines in the debug output
     for byte in &mut buf[debug_start..] {
@@ -324,8 +300,9 @@ fn write_seed_line(buf: &mut Vec<u8>, seed: Seed, shrunken_value: &dyn Debug)
 }
 
 fn write_header(buf: &mut Vec<u8>) -> io::Result<()> {
-    writeln!(buf,
-"\
+    writeln!(
+        buf,
+        "\
 # Seeds for failure cases proptest has generated in the past. It is
 # automatically read and these particular cases re-run before any
 # novel cases are generated.
@@ -362,8 +339,8 @@ impl FileFailurePersistence {
                     let mut dir = Cow::into_owned(source_path.clone());
                     let mut found = false;
                     while dir.pop() {
-                        if dir.join("lib.rs").is_file() ||
-                            dir.join("main.rs").is_file()
+                        if dir.join("lib.rs").is_file()
+                            || dir.join("main.rs").is_file()
                         {
                             found = true;
                             break;
@@ -419,7 +396,9 @@ impl FileFailurePersistence {
 
             Direct(path) => Some(Path::new(path).to_owned()),
 
-            _NonExhaustive => panic!("FailurePersistence set to _NonExhaustive"),
+            _NonExhaustive => {
+                panic!("FailurePersistence set to _NonExhaustive")
+            }
         }
     }
 }
@@ -525,7 +504,8 @@ mod tests {
     fn relative_source_files_absolutified() {
         const TEST_RUNNER_PATH: &[&str] = &["src", "test_runner", "mod.rs"];
         lazy_static! {
-            static ref TEST_RUNNER_RELATIVE: PathBuf = TEST_RUNNER_PATH.iter().collect();
+            static ref TEST_RUNNER_RELATIVE: PathBuf =
+                TEST_RUNNER_PATH.iter().collect();
         }
         const CARGO_DIR: &str = env!("CARGO_MANIFEST_DIR");
 
@@ -539,7 +519,8 @@ mod tests {
             absolutize_source_file_with_cwd(
                 || Ok(Path::new(CARGO_DIR).to_owned()),
                 &TEST_RUNNER_RELATIVE
-            ).unwrap()
+            )
+            .unwrap()
         );
 
         // Running from test subdirectory
@@ -548,7 +529,8 @@ mod tests {
             absolutize_source_file_with_cwd(
                 || Ok(Path::new(CARGO_DIR).join("target")),
                 &TEST_RUNNER_RELATIVE
-            ).unwrap()
+            )
+            .unwrap()
         );
     }
 }
